<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * API configuration form
 *
 * @package     quizaccess_quilgo
 * @copyright   2023 Native Platform Ltd <hello@quilgo.com>
 * @license     http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace quizaccess_quilgo\form;

use html_writer;

defined('MOODLE_INTERNAL') || die();

require_once($CFG->libdir . '/formslib.php');
require_once($CFG->dirroot. '/mod/quiz/accessrule/quilgo/lib.php');

class api_config extends \moodleform {

    /**
     * Define information the API for the plugin configured or not
     * Action for register new site is handled inside config page
     * @return void
     */
    public function definition() {
        $mform = $this->_form;
        $apitoken = get_config('quizaccess_quilgo', CONFIG_API_TOKEN_NAME);
        $clienttoken = get_config('quizaccess_quilgo', CONFIG_CLIENT_TOKEN_NAME);

        if (empty($apitoken) || empty($clienttoken)) {
            $warning = html_writer::tag(
                'div',
                get_string('config_warning_not_active_yet', 'quizaccess_quilgo'),
                ['class' => 'alert alert-warning']
            );

            $mform->addElement('html', $warning);
            $mform->addElement('submit', 'register', get_string('config_register', 'quizaccess_quilgo'));

            return;
        }

        $info = html_writer::tag(
            'div',
            get_string('config_info_active', 'quizaccess_quilgo'),
            ['class' => 'alert alert-info']
        );

        $mform->addElement('html', $info);
    }

}
